-- Table to store universities
CREATE TABLE universities (
    cif VARCHAR(9) PRIMARY KEY NOT NULL,
    name VARCHAR(40) NOT NULL,
    direction VARCHAR(100) NOT NULL
);

-- Table to store companies
CREATE TABLE companies (
    cif VARCHAR(9) PRIMARY KEY NOT NULL,
    name VARCHAR(40) NOT NULL,
    direction VARCHAR(100) NOT NULL,
    validation_date DATE NOT NULL
);

-- Table to store degrees
CREATE TABLE degrees (
    code VARCHAR(20) PRIMARY KEY NOT NULL,
    name VARCHAR(100) NOT NULL,
    area VARCHAR(100) CHECK (area IN ('Education', 'Political Science', 'Law', 
        'Science of Security', 'Economy', 'Marketing and Communication', 
        'Engineering', 'Dessign', 'Medicine', 'Social Sciences', 
        'Art and Humanity', 'Music', 'Science')) NOT NULL
);

-- Table to store the degrees offered by universities
CREATE TABLE degree_offers (
	degree VARCHAR(20) NOT NULL,
	university VARCHAR(9) NOT NULL,
	PRIMARY KEY (degree, university),
	FOREIGN KEY (degree) REFERENCES degrees(code),
	FOREIGN KEY (university) REFERENCES universities(cif)
);

-- Table to store persons
CREATE TABLE persons (
    dni VARCHAR(9) PRIMARY KEY NOT NULL, 
    name VARCHAR(20) NOT NULL,
    surname1 VARCHAR(20) NOT NULL,
    surname2 VARCHAR(20),
    birthdate DATE NOT NULL,
    telf VARCHAR(20) NOT NULL,
    direction VARCHAR(100)  NOT NULL
);

-- Table to store the students
CREATE TABLE students (
    dni VARCHAR(9) PRIMARY KEY NOT NULL,
    code VARCHAR(20) NOT NULL,
    university VARCHAR(9) NOT NULL,
    degree VARCHAR(20) NOT NULL,
    FOREIGN KEY (dni) REFERENCES persons(dni),
    FOREIGN KEY (university) REFERENCES universities(cif),
    FOREIGN KEY (degree) REFERENCES degrees(code),
    UNIQUE (code, university)
);

-- Table to store the professors
CREATE TABLE professors (
	dni VARCHAR(9) PRIMARY KEY NOT NULL,
	email VARCHAR(50) NOT NULL UNIQUE,
	code VARCHAR(20) NOT NULL,
	university VARCHAR(9) NOT NULL,  
	FOREIGN KEY (dni) REFERENCES persons(dni),
	FOREIGN KEY (university) REFERENCES universities(cif),
	UNIQUE (code, university)
);

-- Table to store the responsibles
CREATE TABLE responsibles (
	dni VARCHAR(9) PRIMARY KEY NOT NULL,
	company VARCHAR(9) NOT NULL,
	FOREIGN KEY (dni) REFERENCES persons(dni),
	FOREIGN KEY (company) REFERENCES companies(cif)
);

-- Table to store the human resources contacts
CREATE TABLE RRHH_contacts (
    dni VARCHAR(9) PRIMARY KEY NOT NULL,
    email VARCHAR(50) NOT NULL UNIQUE,
    company VARCHAR(9) NOT NULL,
    FOREIGN KEY (dni) REFERENCES persons(dni),
    FOREIGN KEY (company) REFERENCES companies(cif)
);

-- Table to store the offers made it by companies
CREATE TABLE offers (
    idOffer VARCHAR(10) PRIMARY KEY NOT NULL,
    name VARCHAR(50) NOT NULL,
    description VARCHAR(500) NOT NULL,
    publish_date DATE NOT NULL,
    company VARCHAR(9) NOT NULL,
    university VARCHAR(9) NOT NULL,
    FOREIGN KEY (company) REFERENCES companies(cif),
    FOREIGN KEY (university) REFERENCES universities(cif)
);

-- Table to store the student's candidacies
CREATE TABLE candidacies (
    idCandidacy VARCHAR(10) PRIMARY KEY NOT NULL,
    valid NUMBER(1),
    validation_date DATE,
    state VARCHAR(100) CHECK (state IN ('Presented', 'Validated', 
        'Interview', 'Accepted', 'Rejected')) NOT NULL,
    offer VARCHAR(10) NOT NULL,
    student VARCHAR(9) NOT NULL,
    responsible VARCHAR(9) NOT NULL,
    FOREIGN KEY (offer) REFERENCES offers(idOffer),
    FOREIGN KEY (student) REFERENCES students(dni),
    FOREIGN KEY (responsible) REFERENCES responsibles(dni)
);

-- Table to store the requisites of the presented candidacies
CREATE TABLE candidacy_requisites (
    idRequisite VARCHAR(10) PRIMARY KEY NOT NULL,
    requisite VARCHAR(500) NOT NULL,
    candidacy VARCHAR(10) NOT NULL,
    FOREIGN KEY (candidacy) REFERENCES candidacies(idCandidacy)
);

-- Table to store the interviews
CREATE TABLE interviews (
    idInterview VARCHAR(10) PRIMARY KEY NOT NULL,
    interview_date DATE NOT NULL,
    valid NUMBER(1),
    candidacy VARCHAR(10) UNIQUE NOT NULL,
    rrhh_contact VARCHAR(9) NOT NULL,
    FOREIGN KEY (candidacy) REFERENCES candidacies(idCandidacy),
    FOREIGN KEY (rrhh_contact) REFERENCES RRHH_contacts(dni)
);

-- Table to store comments of every interview
CREATE TABLE interview_comments (
    idComment VARCHAR(10) PRIMARY KEY NOT NULL,
    interview_comment VARCHAR(500) NOT NULL,
    interview VARCHAR(10) NOT NULL,
    FOREIGN KEY (interview) REFERENCES interviews(idInterview)
);

-- Table to store the collaborations
CREATE TABLE collaborations (
    idCollaboration VARCHAR(10) PRIMARY KEY NOT NULL,
    init_date DATE NOT NULL,
    duration NUMBER(10) NOT NULL,
    remuneration NUMBER(4) NOT NULL,
    periodic_report_periodicity NUMBER(2) NOT NULL,
    student VARCHAR(9) NOT NULL,
    responsible VARCHAR(9) NOT NULL,
    professor VARCHAR(9) NOT NULL,
    FOREIGN KEY (student) REFERENCES students(dni),
    FOREIGN KEY (responsible) REFERENCES responsibles(dni),
    FOREIGN KEY (professor) REFERENCES professors(dni)    
);

-- Table to store the collaboration's themes
CREATE TABLE collaboration_themes (
    idTheme VARCHAR(12) PRIMARY KEY NOT NULL,
    name VARCHAR(10) NOT NULL,
    description VARCHAR(500) NOT NULL,
    collaboration VARCHAR(10) NOT NULL,
    FOREIGN KEY (collaboration) REFERENCES collaborations(idCollaboration)
);

-- Table to store the periodic reports
CREATE TABLE periodic_reports (
    idReport VARCHAR(10) PRIMARY KEY NOT NULL,
    report_date DATE NOT NULL,
    report_comment VARCHAR(500) NOT NULL,
    collaboration VARCHAR(10) NOT NULL,
    FOREIGN KEY (collaboration) REFERENCES collaborations(idCollaboration)
);

-- Table to store the inspections
CREATE TABLE inspections (
    idInspection VARCHAR(10) PRIMARY KEY NOT NULL,
    inspection_date DATE NOT NULL,
    passed NUMBER(1) NOT NULL,
    collaboration VARCHAR(10) NOT NULL,
    FOREIGN KEY (collaboration) REFERENCES collaborations(idCollaboration)
);

-- Table to store the final reports
CREATE TABLE final_reports (
    idReport VARCHAR(10) PRIMARY KEY NOT NULL,
    report_date DATE NOT NULL,
    valoration NUMBER(4, 2) NOT NULL,
    suitable NUMBER(1) NOT NULL,
    collaboration VARCHAR(10) NOT NULL,
    FOREIGN KEY (collaboration) REFERENCES collaborations(idCollaboration)
);

-- Table to store the final report's comments
CREATE TABLE final_report_comments (
    idComment VARCHAR(10) PRIMARY KEY NOT NULL,
    final_report_comment VARCHAR(500) NOT NULL,
    final_report VARCHAR(10),
    person VARCHAR(9),
    FOREIGN KEY (final_report) REFERENCES final_reports(idReport),
    FOREIGN KEY (person) REFERENCES persons(dni)
);

-- Table to store the logs
CREATE TABLE log_storeds (
    idLog VARCHAR(15) PRIMARY KEY NOT NULL,
    log_date TIMESTAMP NOT NULL,
    log_process VARCHAR(100) NOT NULL,
    log_entry VARCHAR(200) NOT NULL,
    log_return VARCHAR(200) NOT NULL
);

-- This table contai general counters to help with the stadistic repository
CREATE TABLE general_counters (
	counter_name VARCHAR(30) PRIMARY KEY NOT NULL,
	counter_value NUMBER DEFAULT 0
);
/

-- Insert one by one all the general_counters
INSERT INTO general_counters (counter_name)
VALUES ('university_counter');
/
INSERT INTO general_counters (counter_name)
VALUES ('collaborations_counter');
/
INSERT INTO general_counters (counter_name)
VALUES ('aggregated_duration');
/
INSERT INTO general_counters (counter_name)
VALUES ('company_counter');
/
INSERT INTO general_counters (counter_name)
VALUES ('comp_with_off_last');
/
INSERT INTO general_counters (counter_name)
VALUES ('comp_with_off_last_not_current');
/
INSERT INTO general_counters (counter_name)
VALUES ('comp_with_off_current');
/
INSERT INTO general_counters (counter_name)
VALUES ('counter_uni_collab_this_year');
/

-- This table contains counters, boolean and partial results to diferent estadistics about academic_years
CREATE TABLE university_year_stats_aux (
	academic_year VARCHAR(10) PRIMARY KEY NOT NULL,
	total_offers NUMBER DEFAULT 0,
	total_candidacies NUMBER DEFAULT 0,
	total_collaborations NUMBER DEFAULT 0,
	aggregated_income NUMBER DEFAULT 0,
	mean_income NUMBER DEFAULT 0,
	total_collaborations_ended NUMBER DEFAULT 0,
	suitable_collaborations NUMBER DEFAULT 0,
	non_suitable_collaborations NUMBER DEFAULT 0,
	non_suitable_percentage NUMBER DEFAULT 0
);
/

-- This table contains counters, boolean and partial results to different estadistics about companies
CREATE TABLE companies_stats_aux (
    cif VARCHAR(9) PRIMARY KEY NOT NULL,
	total_offers NUMBER DEFAULT 0,
    total_collaborations_started NUMBER DEFAULT 0,
	total_collaborations_ended NUMBER DEFAULT 0,
    suitable_collaborations NUMBER DEFAULT 0,
	non_suitable_collaborations NUMBER DEFAULT 0,
	non_suitable_percentage NUMBER DEFAULT 0,
	total_inspections NUMBER DEFAULT 0,
	passed_inspections NUMBER DEFAULT 0,
	failed_inspections NUMBER DEFAULT 0,
	failed_inspections_percentage NUMBER DEFAULT 0,
	has_offers_this_year NUMBER(1) DEFAULT 0,
	had_offers_last_year NUMBER(1) DEFAULT 0
);
/

-- This table contains counters, boolean and partial results to different estadistics about universities
CREATE TABLE university_stats_aux (
	cif VARCHAR(9) PRIMARY KEY NOT NULL,
	has_collaboration_this_year NUMBER(1) DEFAULT 0,
	total_collaborations_ended NUMBER DEFAULT 0,
    suitable_collaborations NUMBER DEFAULT 0,
	non_suitable_collaborations NUMBER DEFAULT 0,
	suitable_percentage NUMBER DEFAULT 0
);
/

-- This table contains counters, boolean and partial results to different estadistics about degrees
CREATE TABLE degree_stats_aux (
	code VARCHAR(20) PRIMARY KEY NOT NULL,
	suitable_collaborations_ended NUMBER DEFAULT 0
);
/

-- This table contains counters, boolean and partial results to different estadistics about professors
CREATE TABLE professor_stats_aux (
	dni VARCHAR(9) PRIMARY KEY NOT NULL,
	current_year_collabs_supervised NUMBER DEFAULT 0
);
/

-- This table is neede to implement later the logic that updates the eleventh indicator
CREATE TABLE indicator_11_aux (
	cif VARCHAR(9) PRIMARY KEY NOT NULL
);
/

-- Table to store the indicators that are unique
CREATE TABLE unique_indicators (
    type_indicator NUMBER(2) PRIMARY KEY NOT NULL,
    type_description VARCHAR(500) NOT NULL,
    type_value VARCHAR(50)
);

-- Table to store the second indicator
CREATE TABLE indicator_type_2 (
    company VARCHAR(40)
);

-- Table to store the fourth indicator
CREATE TABLE indicator_type_4 (
    year VARCHAR(20) UNIQUE,
    value NUMBER(5,2)
);

-- Table to store the eleventh indicator
CREATE TABLE indicator_type_11 (
    year VARCHAR(20) UNIQUE,
    company VARCHAR(40)
);

-- Table to store the fifteenth indicator
CREATE TABLE indicator_type_15 (
    company VARCHAR(40)
);

-- Table to store the sixteenth indicator
CREATE TABLE indicator_type_16 (
    company VARCHAR(40)
);

-- We populate the unique_indicators table for work on it later
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (1, 'Universidad con porcentaje más elevado de prácticas superadas.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (3, 'Empresa que ha recibido más inspecciones.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (5, 'Sueldo medio que reciben los estudiantes durante este año.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (6, 'Número de estudiantes que están realizando prácticas.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (7, 'Porcentaje de universidades sin alumnos en prácticas para este año.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (8, 'Titulación universitaria con mayor exito en las prácticas.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (9, 'Número medio de estudiantes que se presentan a cada oferta.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (10, 'Duración media de los contratos en prácticas.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (12, 'Profesor que supervisa más alumnos en el año en curso.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (13, 'Número total de informes de prácticas pendientes de ser realizados.');
INSERT INTO unique_indicators (type_indicator, type_description)
VALUES (14, 'Porcentaje de empresas que ofrecieron prácticas el año anterior y no este.');

-- It creates the sequences that will return the Primary Key of several tables
CREATE SEQUENCE seq_id_offers START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_candidacies START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_candidacy_requisites START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_interviews START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_interview_comments START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_collaborations START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_collaboration_themes START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_periodic_reports START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_inspections START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_final_reports START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_final_report_comments START WITH 1 INCREMENT BY 1;
CREATE SEQUENCE seq_id_log_storeds START WITH 1 INCREMENT BY 1;

-- This procedure is in charge to insert or update a company
CREATE OR REPLACE PROCEDURE insert_or_update_company (
	p_cif IN VARCHAR2,
	p_name IN VARCHAR2,
	p_direction IN VARCHAR2,
	p_validation_date IN DATE,
	RSP OUT VARCHAR2
)
AS
	p_count NUMBER;
	p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
	p_entry_values := p_cif || ', ' || p_name || ', ' || p_direction || ', ' || p_validation_date;
	-- It counts if the passed CIF is currently stored in the table
	SELECT COUNT(*) INTO p_count FROM companies WHERE cif = p_cif;
	-- It checks if the CIF passed as parameter has a exactly length of 9 characters
	IF LENGTH(p_cif) <> 9 THEN
		p_return_value := 'KO: El CIF debe tener exactamente 9 caracteres';
	-- If the cif is not in the table, it creates a new row with the passed data.
	ELSIF (p_count = 0) THEN
		INSERT INTO companies (cif, name, direction, validation_date)
		VALUES (p_cif, p_name, p_direction, p_validation_date);
		p_return_value := 'OK';
	-- If the cif is in the table, it updates it with the new data
	ELSE
		UPDATE companies
		SET name = p_name, direction = p_direction, validation_date = p_validation_date
		WHERE cif = p_cif;
		p_return_value := 'OK';
	END IF;
	-- Save the log
	INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
	VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_or_update_company', p_entry_values, p_return_value);
	RSP := 'insert_or_update_company(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This procedure is in charge to insert or update a university
CREATE OR REPLACE PROCEDURE insert_or_update_university (
	p_cif IN VARCHAR2,
	p_name IN VARCHAR2,
	p_direction IN VARCHAR2,
	RSP OUT VARCHAR2
)
AS
	p_count NUMBER;
	p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
	p_entry_values := p_cif || ', ' || p_name || ', ' || p_direction;
	-- It counts if the passed CIF is currently stored in the table
	SELECT COUNT(*) INTO p_count FROM universities WHERE cif = p_cif;
	-- It checks if the CIF passed as parameter has a exactly length of 9 characters
	IF LENGTH(p_cif) <> 9 THEN
		p_return_value := 'KO: El CIF debe tener exactamente 9 caracteres';
	-- If the cif is not in the table, it creates a new row with the passed data.
	ELSIF (p_count = 0) THEN
		INSERT INTO universities (cif, name, direction)
		VALUES (p_cif, p_name, p_direction);
		p_return_value := 'OK';
	-- If the cif is in the table, it updates it with the new data
	ELSE
		UPDATE universities
		SET name = p_name, direction = p_direction
		WHERE cif = p_cif;
		p_return_value := 'OK';
	END IF;
	-- Save the log
	INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
	VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_or_update_university', p_entry_values, p_return_value);
	RSP := 'insert_or_update_university(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge to create or update a degree
CREATE OR REPLACE PROCEDURE insert_or_update_degree (
    p_code IN VARCHAR2,
    p_name IN VARCHAR2,
    p_area IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_code || ', ' || p_name || ', ' || p_area;
	-- Checks if the degree is currently stored in the table
    SELECT COUNT(*) INTO p_count FROM degrees WHERE code = p_code;
	-- Check if the p_area value passed is correct
    IF (p_area NOT IN ('Education', 'Political Science', 'Law', 
        'Science of Security', 'Economy', 'Marketing and Communication', 
        'Engineering', 'Dessign', 'Medicine', 'Social Sciences', 
        'Art and Humanity', 'Music', 'Science')) THEN
        p_return_value := 'KO: El Área debe ser valida.';
	-- If the p_code is not in the table, creates a new row
    ELSIF (p_count = 0) THEN
        INSERT INTO degrees (code, name, area)
        VALUES (p_code, p_name, p_area);
        p_return_value := 'OK';
	-- If the p_code is in the table, updates it
    ELSE
        UPDATE degrees
        SET name = p_name, area = p_area
        WHERE code = p_code;
        p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_or_update_degree', p_entry_values, p_return_value);
	RSP := 'insert_or_update_degree(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- Insert a new row in degree_offers table
CREATE OR REPLACE PROCEDURE insert_degree_offers (
    p_degree IN VARCHAR2,
    p_university IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_degree_count NUMBER;
    p_university_count NUMBER;
    p_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_degree || ', ' || p_university;
	-- This SELECT COUNT(*) will be uses to check the passed values
    SELECT COUNT(*) INTO p_degree_count FROM degrees WHERE code = p_degree;
    SELECT COUNT(*) INTO p_university_count FROM universities WHERE cif = p_university;
    SELECT COUNT(*) INTO p_count FROM degree_offers WHERE degree = p_degree 
        AND university = p_university;
	-- If the degree doesn't exist, returns an error
    IF p_degree_count = 0 THEN
        p_return_value := 'KO: El grado indicado no existe.';
	-- If the university doesn't exist, returns an error
    ELSIF p_university_count = 0 THEN
        p_return_value := 'KO: La universidad indicada no existe.';
	-- If the combination of degree and university exists in the table, returns an error.
    ELSIF p_count > 0 THEN
        p_return_value := 'KO: Esta combinación ya está incluida.';
	-- If no errors, insert a new row
    ELSE
        INSERT INTO degree_offers (degree, university)
        VALUES (p_degree, p_university);
		p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_degree_offers', p_entry_values, p_return_value);
	RSP := 'insert_degree_offers(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge of insert or update student
CREATE OR REPLACE PROCEDURE insert_or_update_student (
	p_dni IN VARCHAR2,
	p_name IN VARCHAR2,
	p_surname1 IN VARCHAR2,
	p_surname2 IN VARCHAR2,
	p_birthdate IN DATE,
	p_telf IN VARCHAR2,
	p_direction IN VARCHAR2,
	p_code IN VARCHAR2,
	p_university IN VARCHAR2,
	p_degree IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_count NUMBER;
    p_degree_count NUMBER;
    p_university_count NUMBER;
	p_code_count NUMBER;
	p_university_has_degree NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_dni || ', ' || p_name || ', ' || p_surname1 || ', ' || 
    p_surname2 || ', ' || p_birthdate || ', ' || p_telf || ', ' || 
    p_direction || ', ' || p_code || ', ' || p_university || ', ' || p_degree;
	-- This SELECT COUNT(*) will be uses to check the passed values
    SELECT COUNT(*) INTO p_count FROM persons WHERE dni = p_dni;
    SELECT COUNT(*) INTO p_degree_count FROM degrees WHERE code = p_degree;
    SELECT COUNT(*) INTO p_university_count FROM universities WHERE cif = p_university;
	SELECT COUNT(*) INTO p_university_has_degree FROM degree_offers WHERE degree = p_degree AND university = p_university;
	SELECT COUNT(*) INTO p_code_count FROM students WHERE p_code = code AND p_university = university;
	-- If the degree doesn't exist, returns an error
    IF p_degree_count = 0 THEN
        p_return_value := 'KO: El Grado indicado no existe.';
	-- If the university doesn't exist, returns an error
    ELSIF p_university_count = 0 THEN
        p_return_value := 'KO: La Universidad indicada no existe.';
	-- If the university hadn't the degree returns an error
	ELSIF p_university_has_degree = 0 THEN
		p_return_value := 'KO: La Universidad no ofrece los estudios indicados.';
	-- If the student doesn't exist, it inserts a new Person and a new Student
    ELSIF p_count = 0 THEN
		IF p_code_count = 0 THEN
			INSERT INTO persons (dni, name, surname1, surname2, birthdate, telf, direction)
			VALUES(p_dni, p_name, p_surname1, p_surname2, p_birthdate, p_telf, p_direction);
			INSERT INTO students (dni, code, university, degree)
			VALUES(p_dni, p_code, p_university, p_degree);
			p_return_value := 'OK';
		-- If the code is in use, returns an error
		ELSE
			p_return_value := 'KO: Este código ya está en uso en esta universidad.';
		END IF;
	-- If exists, updates the current ones.
    ELSE
		IF p_code_count > 0 THEN
			UPDATE persons
			SET name = p_name, surname1 = p_surname1, surname2 = p_surname2, birthdate = p_birthdate, telf = p_telf, direction = p_direction
			WHERE dni = p_dni;
			UPDATE students
			SET code = p_code, university = p_university, degree = p_degree
			WHERE dni = p_dni;
			p_return_value := 'OK';
		-- IF we are trying to change the code, returns an error.
		ELSE 
			p_return_value := 'KO: El código es unico y no se puede cambiar.';
		END IF;
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_or_update_student', p_entry_values, p_return_value);
	RSP := 'insert_or_update_student(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge of insert or update a professor
CREATE OR REPLACE PROCEDURE insert_or_update_professor (
    p_dni IN VARCHAR2,
    p_name IN VARCHAR2,
    p_surname1 IN VARCHAR2,
    p_surname2 IN VARCHAR2,
    p_birthdate IN DATE,
    p_telf IN VARCHAR2,
    p_direction IN VARCHAR2,
    p_email IN VARCHAR2,
    p_code IN VARCHAR2,
    p_university IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_count NUMBER;
    p_university_count NUMBER;
	p_email_count_1 NUMBER;
	p_email_count_2 NUMBER;
	p_code_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_dni || ', ' || p_name || ', ' || p_surname1 || ', ' || 
    p_surname2 || ', ' || p_birthdate || ', ' || p_telf || ', ' || 
    p_direction || ', ' || p_email || ', ' ||p_code || ', ' || p_university;
	-- This SELECT COUNT(*) will be uses to check the passed values
    SELECT COUNT(*) INTO p_count FROM persons WHERE dni = p_dni;
    SELECT COUNT(*) INTO p_university_count FROM universities WHERE cif = p_university;
	SELECT COUNT(*) INTO p_email_count_1 FROM Professors WHERE email = p_email;
	SELECT COUNT(*) INTO p_email_count_2 FROM RRHH_contacts WHERE email = p_email;
	SELECT COUNT(*) INTO p_code_count FROM Professors WHERE p_code = code AND p_university = university;
	-- If the university doesn't exist, returns an error
    IF p_university_count = 0 THEN
        p_return_value := 'KO: La Universidad indicada no existe.';	
	-- If the professor doesn't exist, it inserts a new Person and a new Professor
    ELSIF p_count = 0 THEN
		-- If the e-mail is in some table, it will return an error
		IF (p_email_count_1 + p_email_count_2) > 0 THEN
			p_return_value := 'KO: Este e-mail ya está en la base de datos.';
		-- If the code is in use on that university then return an error.
		ELSIF p_code_count > 0 THEN
			p_return_value := 'KO: Este código ya está registrado en esta universidad.';
		ELSE
			INSERT INTO persons (dni, name, surname1, surname2, birthdate, telf, direction)
			VALUES(p_dni, p_name, p_surname1, p_surname2, p_birthdate, p_telf, p_direction);
			INSERT INTO professors (dni, email, code, university)
			VALUES(p_dni, p_email, p_code, p_university);
			p_return_value := 'OK';
		END IF;
	-- If exists, updates the current ones.
    ELSE		
		-- If we are not changing the code, proceeds with the update.
		IF p_code_count > 0 THEN
			UPDATE persons
			SET name = p_name, surname1 = p_surname1, surname2 = p_surname2, birthdate = p_birthdate, telf = p_telf, direction = p_direction
			WHERE dni = p_dni;
			UPDATE professors
			SET email = p_email, code = p_code, university = p_university
			WHERE dni = p_dni;
			p_return_value := 'OK';
		-- If we are trying to change the code, returns an error.
		ELSE
			p_return_value := 'KO: El código es único y no se puede cambiar.';
		END IF;
    END IF;    
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_or_update_professor', p_entry_values, p_return_value);
	RSP := 'insert_or_update_professor(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge of insert or update a responsible
CREATE OR REPLACE PROCEDURE insert_or_update_responsible (
    p_dni IN VARCHAR2,
    p_name IN VARCHAR2,
    p_surname1 IN VARCHAR2,
    p_surname2 IN VARCHAR2,
    p_birthdate IN DATE,
    p_telf IN VARCHAR2,
    p_direction IN VARCHAR2,
    p_company IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_count NUMBER;
    p_company_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_dni || ', ' || p_name || ', ' || p_surname1 || ', ' || 
    p_surname2 || ', ' || p_birthdate || ', ' || p_telf || ', ' || 
    p_direction || ', ' || p_company;
	-- This SELECT COUNT(*) will be uses to check the passed values
    SELECT COUNT(*) INTO p_count FROM persons WHERE dni = p_dni;
    SELECT COUNT(*) INTO p_company_count FROM companies WHERE cif = p_company;
	-- If the company doesn't exist, return an error
    IF p_company_count = 0 THEN
        p_return_value := 'KO: La Compañia indicada no existe.';
	-- If the responsible doesn't exist, it inserts a new Person and a new responsible
    ELSIF p_count = 0 THEN
        INSERT INTO persons (dni, name, surname1, surname2, birthdate, telf, direction)
        VALUES(p_dni, p_name, p_surname1, p_surname2, p_birthdate, p_telf, p_direction);
        INSERT INTO responsibles (dni, company)
        VALUES(p_dni, p_company);
        p_return_value := 'OK';
	-- If exists, updates the current one.
    ELSE
        UPDATE persons
        SET name = p_name, surname1 = p_surname1, surname2 = p_surname2, birthdate = p_birthdate, telf = p_telf, direction = p_direction
        WHERE dni = p_dni;
        UPDATE responsibles
        SET company = p_company
        WHERE dni = p_dni;
        p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_or_update_responsible', p_entry_values, p_return_value);
	RSP := 'insert_or_update_responsible(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge of update or insert a RRHH_contact
CREATE OR REPLACE PROCEDURE insert_or_update_RRHH_contact (
    p_dni IN VARCHAR2,
    p_name IN VARCHAR2,
    p_surname1 IN VARCHAR2,
    p_surname2 IN VARCHAR2,
    p_birthdate IN DATE,
    p_telf IN VARCHAR2,
    p_direction IN VARCHAR2,
    p_email IN VARCHAR2,
    p_company IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_count NUMBER;
    p_company_count NUMBER;
	p_email_count_1 NUMBER;
	p_email_count_2 NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_dni || ', ' || p_name || ', ' || p_surname1 || ', ' || 
    p_surname2 || ', ' || p_birthdate || ', ' || p_telf || ', ' || 
    p_direction || ', ' || p_email || ', ' || p_company;
	-- This SELECT COUNT(*) will be uses to check the passed values
    SELECT COUNT(*) INTO p_count FROM persons WHERE dni = p_dni;
    SELECT COUNT(*) INTO p_company_count FROM companies WHERE cif = p_company;
	SELECT COUNT(*) INTO p_email_count_1 FROM Professors WHERE email = p_email;
	SELECT COUNT(*) INTO p_email_count_2 FROM RRHH_contacts WHERE email = p_email;
	-- If the company doesn't exist, returns an error
    IF p_company_count = 0 THEN
        p_return_value := 'KO: La Compañia indicada no existe.';	
	-- If the contact doesn't exist, it inserts a new Person and a new Contact
    ELSIF p_count = 0 THEN
		-- If the e-mail is in some table, it will return an error
		IF (p_email_count_1 + p_email_count_2) > 0 THEN
			p_return_value := 'KO: Este e-mail ya está en la base de datos.';
		ELSE
			INSERT INTO persons (dni, name, surname1, surname2, birthdate, telf, direction)
			VALUES(p_dni, p_name, p_surname1, p_surname2, p_birthdate, p_telf, p_direction);
			INSERT INTO RRHH_contacts (dni, email, company)
			VALUES(p_dni, p_email, p_company);
			p_return_value := 'OK';
		END IF;        
	-- If exists, updates the current ones.
    ELSE
        UPDATE persons
        SET name = p_name, surname1 = p_surname1, surname2 = p_surname2, birthdate = p_birthdate, telf = p_telf, direction = p_direction
        WHERE dni = p_dni;
        UPDATE RRHH_contacts
        SET email = p_email, company = p_company
        WHERE dni = p_dni;
        p_return_value := 'OK';
	-- Save the log
    END IF;    
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_or_update_RRHH_contact', p_entry_values, p_return_value);
	RSP := 'insert_or_update_RRHH_contact(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge of insert a new offer
CREATE OR REPLACE PROCEDURE insert_offers (
	p_name IN VARCHAR2,
	p_description IN VARCHAR2,
	p_publish_date IN DATE,
	p_company IN VARCHAR2,
	p_university IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_company_count NUMBER;
    p_university_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_name || ', ' || p_description || ', ' || 
    p_publish_date || ', ' || p_company || ', ' || p_university;
	-- This SELECT COUNT(*) will be used to check if the passed parameters are ok
    SELECT COUNT(*) INTO p_university_count FROM universities WHERE cif = p_university;
    SELECT COUNT(*) INTO p_company_count FROM companies WHERE cif = p_company;
	-- If the university doesn't exist, returns an error
    IF p_university_count = 0 THEN
        p_return_value := 'KO: La Universidad indicada no existe.';
	-- If the company doesn't exist, returns an error
    ELSIF p_company_count = 0 THEN
        p_return_value := 'KO: La Compañia indicada no existe.';
	-- If everything is ok, insert a new offer
    ELSE
        INSERT INTO offers (idOffer, name, description, publish_date, company, university)
        VALUES (seq_id_offers.NEXTVAL, p_name, p_description, p_publish_date, p_company, p_university);
        p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_offers', p_entry_values, p_return_value);
	RSP := 'insert_offers(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This procedure is in charge to insert a new candidacy
CREATE OR REPLACE PROCEDURE insert_new_candidacy (
    p_offer IN VARCHAR2,
    p_student IN VARCHAR2,
    p_responsible IN VARCHAR2,
    RSP OUT VARCHAR2 
)
AS
    p_student_count NUMBER;
	p_student_count_2 NUMBER;
    p_offer_count NUMBER;
    p_responsible_count NUMBER;
	p_responsible_count_2 NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_offer || ', ' || p_student || ', ' || p_responsible;
	-- This SELECT COUNT(*) will be used to check if the passed parameters are ok
    SELECT COUNT(*) INTO p_student_count FROM students WHERE dni = p_student;
	SELECT COUNT(*) INTO p_student_count_2 FROM students S, offers O
		WHERE O.university = S.university
			AND p_offer = O.idOffer
			AND p_student = S.dni;
    SELECT COUNT(*) INTO p_offer_count FROM offers WHERE  idOffer = p_offer;
    SELECT COUNT(*) INTO p_responsible_count FROM responsibles WHERE dni = p_responsible;
	SELECT COUNT(*) INTO p_responsible_count_2 FROM responsibles R, offers O 
		WHERE O.company = R.company 
			AND p_offer = O.idOffer 
			AND p_responsible = R.dni;
	-- If the student doesn't exist, returns an error
    IF p_student_count = 0 THEN
        p_return_value := 'KO: El estudiante indicado no existe.';
	-- If the offer doesn't exist, returns an error
    ELSIF p_offer_count = 0 THEN
        p_return_value := 'KO: La oferta indicada no existe.';
	-- If the responsible doesn't exist, returns an error
    ELSIF p_responsible_count = 0 THEN
        p_return_value := 'KO: El responsible indicado no existe.';
	ELSIF p_responsible_count_2 = 0 THEN
		p_return_value := 'KO: El responsible indicado no trabaja en la empresa que emite la oferta.';
	ELSIF p_student_count_2 = 0 THEN
		p_return_value := 'KO: El estudiante indicado no estudia en la universidad de la oferta.';
	-- If everything is ok, creates a new candidacy
    ELSE
        INSERT INTO candidacies (idCandidacy, valid, validation_date, state, offer, student, responsible)
        VALUES (seq_id_candidacies.NEXTVAL, NULL, NULL, 'Presented', p_offer, p_student, p_responsible);
        p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_new_candidacy', p_entry_values, p_return_value);
	RSP := 'insert_new_candidacy(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This procedure saves the every requisites that the students send for its candidacies
CREATE OR REPLACE PROCEDURE insert_candidacy_requisites (
	p_requisite IN VARCHAR2,
	p_candidacy IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_candidacy_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_requisite || ', ' || p_candidacy;
	-- Firts of all, check if the candidacy exist. If not, returns an error.
    SELECT COUNT(*) INTO p_candidacy_count FROM candidacies WHERE idCandidacy = p_candidacy;
    IF p_candidacy_count = 0 THEN
        p_return_value := 'KO: La candidatura indicada no existe.';
	-- If the candidacy exists, it creates the requisite
    ELSE
        INSERT INTO candidacy_requisites (idRequisite, requisite, candidacy)
        VALUES(seq_id_candidacy_requisites.NEXTVAL, p_requisite, p_candidacy);
        p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_candidacy_requisites', p_entry_values, p_return_value);
	RSP := 'insert_candidacy_requisites(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This procedure is in charge to modify a candidature state
CREATE OR REPLACE PROCEDURE change_candidacy_state (
	p_candidacy IN VARCHAR2,
	p_state IN VARCHAR2,
	p_return OUT NUMBER
)
AS
    p_candidacy_count_1 NUMBER;
	p_candidacy_count_2 NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_candidacy || ', ' || p_state;
	SELECT COUNT(*) INTO p_candidacy_count_1 FROM candidacies WHERE idCandidacy = p_candidacy;
	SELECT COUNT(*) INTO p_candidacy_count_2 FROM candidacies C WHERE C.idCandidacy = p_candidacy AND C.state IN ('Accepted', 'Rejected');
	-- Checks if the passed state is correct. If not, returns an error.
    IF (p_state NOT IN ('Presented', 'Validated', 'Interview', 
                        'Accepted', 'Rejected')) THEN
        p_return_value := 'KO: El estado indicado no es correcto.';	
	-- Checks if the candidacy exists. If not, returns an error.
    ELSIF p_candidacy_count_1 = 0 THEN
        p_return_value := 'KO: La candidatura indicada no existe.';
	-- Checks if the candidacy is modificable. If not, returns an error.
	ELSIF p_candidacy_count_2 > 0 THEN
		p_return_value := 'KO: La candidatura indicada ya no se puede editar.';
		p_return := 0;
	-- If no errors, then change the state of the candidacy
    ELSE
        UPDATE candidacies
        SET state = p_state
        WHERE idCandidacy = p_candidacy;
        p_return_value := 'OK'; 
		p_return := 1;
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'change_candidacy_state', p_entry_values, p_return_value);    
END;
/

-- This procedure is the one who validate the candidacy
CREATE OR REPLACE PROCEDURE validate_candidacy (
    p_candidacy IN VARCHAR2,
    p_valid IN NUMBER,
    p_date IN DATE,
    RSP OUT VARCHAR2
)
AS
    p_candidacy_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
    p_candidacy_return NUMBER;
BEGIN
    p_entry_values := p_candidacy || ', ' || p_valid || ', ' || p_date;
	-- Checks if the candidacy exists. If not, return and error.
    SELECT COUNT(*) INTO p_candidacy_count FROM candidacies WHERE idCandidacy = p_candidacy;
    IF p_candidacy_count = 0 THEN
        p_return_value := 'KO: La candidatura indicada no existe.';
	-- Set the Candidacy to Validated
    ELSIF p_valid = 1 THEN
		change_candidacy_state(p_candidacy, 'Validated', p_candidacy_return);
		IF p_candidacy_return = 1 THEN
			UPDATE candidacies
			SET valid = p_valid,  validation_date = p_date
			WHERE idCandidacy = p_candidacy;        
			p_return_value := 'OK';
		ELSE
			p_return_value := 'KO: No se puede validar una candidatura que ya está rechazada o aceptada.';
		END IF;        
	-- Set the Candidacy to Rejected
    ELSIF p_valid = 0 THEN
		change_candidacy_state(p_candidacy, 'Rejected', p_candidacy_return);
		IF p_candidacy_return = 1 THEN
			UPDATE candidacies
			SET valid = p_valid, validation_date = p_date
			WHERE idCandidacy = p_candidacy;        
			p_return_value := 'OK';
		ELSE
			p_return_value := 'KO: No se puede validar una candidatura que ya está rechazada o aceptada.';
		END IF;        
	-- If the number passed it's neither 0 nor 1 then returns an error
    ELSE
        p_return_value := 'KO: El valor indicado no es valido.';        
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'validate_candidacy', p_entry_values, p_return_value);
	RSP := 'validate_candidacy(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/
 
-- This stored procedure is in charge to insert a new interview
CREATE OR REPLACE PROCEDURE insert_new_interview (
	p_date IN DATE,
	p_candidacy IN VARCHAR2,
	p_rrhh_contact IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_candidacy_count NUMBER;
    p_rrhh_contact_count NUMBER;
	p_rrhh_contact_count_2 NUMBER;
	p_interview_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
    p_candidacy_return NUMBER;
BEGIN
    p_entry_values := p_date || ', ' || p_candidacy || ', ' || p_rrhh_contact;
	-- This SELECT COUNT(*) will be used to check if the passed parameters are ok
    SELECT COUNT(*) INTO p_candidacy_count FROM candidacies WHERE idCandidacy = p_candidacy;
    SELECT COUNT(*) INTO p_rrhh_contact_count FROM RRHH_contacts WHERE dni = p_rrhh_contact;
	SELECT COUNT(*) INTO p_interview_count FROM interviews WHERE candidacy = p_candidacy;
	SELECT COUNT(*) INTO p_rrhh_contact_count_2 FROM candidacies C
		INNER JOIN offers O ON O.idOffer = C.offer
		INNER JOIN companies C ON C.cif = O.company
		INNER JOIN RRHH_contacts RH ON RH.company = C.cif
		WHERE p_rrhh_contact = RH.dni
			AND p_candidacy = C.idCandidacy;
	-- Return an error if the candidacy doesn't exist
    IF p_candidacy_count = 0 THEN
        p_return_value := 'KO: La candidatura indicada no existe.';
	-- Return an error if the RRHH Contact doesn't exist
    ELSIF p_rrhh_contact_count = 0 THEN
        p_return_value := 'KO: El contacto de RRHH indicado no existe.';
	-- Return an error if the candidacy have an interview
	ELSIF p_interview_count > 0 THEN
		p_return_value := 'KO: Esta candidatura ya tiene una entrevista.';
	-- Return an error if the RRHH Contact doesn't works in the company that supervises that candidacy
	ELSIF p_rrhh_contact_count_2 = 0 THEN
		p_return_value := 'KO: El contacto de recursos humanos indicado no trabaja en la empresa que supervisa esta candidatura.';
	-- If no errors, insert the new interview and set the state of the candidacy to interview
    ELSE
		change_candidacy_state(p_candidacy, 'Interview', p_candidacy_return);
		IF p_candidacy_return = 1 THEN
			INSERT INTO interviews (idInterview, interview_date, valid, candidacy, rrhh_contact)
			VALUES (seq_id_interviews.NEXTVAL, p_date, NULL, p_candidacy, p_rrhh_contact); 			
			p_return_value := 'OK';
		ELSE
			p_return_value := 'KO: No se puede concertar una entrevista a una candidatura aceptada o rechazada.';
		END IF;
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_new_interview', p_entry_values, p_return_value);
	RSP := 'insert_new_interview(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge to insert comments from interviews
CREATE OR REPLACE PROCEDURE insert_interview_comment (
    p_comment IN VARCHAR2,
    p_interview in VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_interview_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_comment || ', ' || p_interview;
	-- If the interview doesn't exist, return an error
    SELECT COUNT(*) INTO p_interview_count FROM interviews WHERE idInterview = p_interview;
    IF p_interview_count = 0 THEN
        p_return_value := 'KO: La entrevista indicada no existe.';
	-- Insert the comment
    ELSE
        INSERT INTO interview_comments(idComment, interview_comment, interview)
        VALUES(seq_id_interview_comments.NEXTVAL, p_comment, p_interview);
        p_return_value := 'OK';
    END IF;    
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_interview_comment', p_entry_values, p_return_value);
	RSP := 'insert_interview_comment(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge to change, for the last time, the state of the candidacy
CREATE OR REPLACE PROCEDURE evaluate_interview (
    p_interview IN VARCHAR2,
    p_valid IN NUMBER,
    RSP OUT VARCHAR2
)
AS
    p_candidacy VARCHAR(10);
    p_interview_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
	p_candidacy_return NUMBER;
BEGIN
    p_entry_values := p_interview || ', ' || p_valid;
    SELECT COUNT(*) INTO p_interview_count FROM interviews WHERE idInterview = p_interview;
	-- Returns an error if the interview doesn't exist
    IF p_interview_count = 0 THEN
        p_return_value := 'KO: La entrevista indicada no existe.';
	-- Set the candidacy to Accepted
    ELSIF p_valid = 1 THEN  
		SELECT candidacy INTO p_candidacy FROM interviews WHERE idInterview = p_interview;
		change_candidacy_state(p_candidacy, 'Accepted', p_candidacy_return);
		IF p_candidacy_return = 1 THEN			
			UPDATE interviews
			SET valid = p_valid
			WHERE idInterview = p_interview;        
			p_return_value := 'OK';
		ELSE
			p_return_value := 'KO: No se puede evaluar una entrevista que ya se ha evaluado.';
		END IF;
	-- Set the candidacy to rejected
    ELSIF p_valid = 0 THEN
		SELECT candidacy INTO p_candidacy FROM interviews WHERE idInterview = p_interview;
		change_candidacy_state(p_candidacy, 'Rejected', p_candidacy_return);
		IF p_candidacy_return = 1 THEN			
			UPDATE interviews
			SET valid = p_valid
			WHERE idInterview = p_interview;
			p_return_value := 'OK';
		ELSE
			p_return_value := 'KO: No se puede evaluar una entrevista que ya se ha evaluado.';
		END IF;
	-- This error implies that the value passed is not 0 or 1
    ELSE
        p_return_value := 'KO: El valor indicado no es valido.'; 
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'evaluate_interview', p_entry_values, p_return_value);
	RSP := 'evaluate_interview(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedue is in charge to insert collaborations
CREATE OR REPLACE PROCEDURE insert_new_collaboration (
	p_init_date IN DATE,
	p_duration IN NUMBER,
	p_remuneration IN NUMBER,
	p_periodic_report_periodicity IN NUMBER,
	p_student IN VARCHAR2,
	p_responsible IN VARCHAR2,
	p_professor IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_student_count NUMBER;
    p_responsible_count NUMBER;
    p_professor_count NUMBER;
	p_university_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_init_date || ', ' || p_duration || ', ' || 
    p_remuneration || ', ' || p_periodic_report_periodicity || ', ' || 
    p_student || ', ' || p_responsible || ', ' || p_professor;
	-- This SELECT COUNT(*) will be used to check if the passed parameters are ok
    SELECT COUNT(*) INTO p_student_count FROM students WHERE dni = p_student;
    SELECT COUNT(*) INTO p_responsible_count FROM responsibles WHERE dni = p_responsible;
    SELECT COUNT(*) INTO p_professor_count FROM professors WHERE dni = p_professor;
	SELECT COUNT(*) INTO p_university_count FROM professors P 
		INNER JOIN universities U ON P.university = U.cif
		INNER JOIN students S ON S.university = U.cif
		WHERE S.dni = p_student AND P.dni = p_professor;
	-- If the student doesn't exist, return a error
    IF p_student_count = 0 THEN
        p_return_value := 'KO: El estudiante indicado no es correcto.';
	-- If the responsible doesn't exist, return a error
    ELSIF p_responsible_count = 0 THEN
        p_return_value := 'KO: El responsable indicado no es correcto.';
	-- If the professor doesn't exist, return a error
    ELSIF p_professor_count = 0 THEN
        p_return_value := 'KO: El profesor indicado no es correcto.';
	-- If the professor and the student are not from the same university, return an error
	ELSIF p_university_count = 0 THEN
		p_return_value := 'KO: El profesor no es de la misma universidad que el estudiante.';
	-- If no errors, create the new collaboration.
    ELSE
        INSERT INTO collaborations (idCollaboration, init_date, duration, 
                                    remuneration, periodic_report_periodicity, 
                                    student, responsible, professor)
        VALUES (seq_id_collaborations.NEXTVAL, p_init_date, p_duration, 
                p_remuneration, p_periodic_report_periodicity, p_student, 
                p_responsible, p_professor);
        p_return_value := 'OK';
    END IF;    
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_new_collaboration', p_entry_values, p_return_value);
	RSP := 'insert_new_collaboration(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This procedure is in charge to insert a new collaboration theme
CREATE OR REPLACE PROCEDURE insert_collaboration_theme (
	p_name IN VARCHAR2,
	p_description IN VARCHAR2,
	p_collaboration IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_collaboration_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_name || ', ' || p_description || ', ' || p_collaboration;
    SELECT COUNT(*) INTO p_collaboration_count FROM collaborations WHERE idCollaboration = p_collaboration;
	-- Return an error if the collaboration doesn't exist
    IF p_collaboration_count = 0 THEN
        p_return_value := 'KO: La colaboración indicada no es correcta.';
	-- Insert the theme if no errors
    ELSE
        INSERT INTO collaboration_themes(idTheme, name, description, collaboration)
        VALUES(seq_id_collaboration_themes.NEXTVAL, p_name, p_description, p_collaboration);
        p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_collaboration_theme', p_entry_values, p_return_value);
	RSP := 'insert_collaboration_theme(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;
END;
/

-- This stored procedure is in charge to store a new periodic report
CREATE OR REPLACE PROCEDURE insert_periodic_report (
	p_report_date IN DATE,
	p_report_comment IN VARCHAR2,
	p_collaboration IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_collaboration_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_report_date || ', ' || p_report_comment || ', ' || p_collaboration;
    SELECT COUNT(*) INTO p_collaboration_count FROM collaborations WHERE idCollaboration = p_collaboration;
	-- Returns an error if the collaboration doesn't exist
    IF p_collaboration_count = 0 THEN
        p_return_value := 'KO: La colaboración indicada no es correcta.';
	-- Insert the report if no errors
    ELSE
        INSERT INTO periodic_reports(idReport, report_date, report_comment, collaboration)
        VALUES(seq_id_periodic_reports.NEXTVAL, p_report_date, p_report_comment, p_collaboration);
        p_return_value := 'OK';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_periodic_report', p_entry_values, p_return_value);
	RSP := 'insert_periodic_report(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;	
END;
/

-- This stored procedure is in charge to insert a new inspection
CREATE OR REPLACE PROCEDURE insert_new_inspection (
	p_inspection_date IN DATE,
	p_passed IN NUMBER,
	p_collaboration IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_collaboration_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_inspection_date || ', ' || p_passed || ', ' || p_collaboration;
	-- If the collaboration doesn't exist, returns an error
    SELECT COUNT(*) INTO p_collaboration_count FROM collaborations WHERE idCollaboration = p_collaboration;
    IF p_collaboration_count = 0 THEN
        p_return_value := 'KO: La colaboración indicada no es correcta.';
	-- Insert a passed inspection
    ELSIF p_passed = 1 THEN
        INSERT INTO inspections(idInspection, inspection_date, passed, collaboration)
        VALUES(seq_id_inspections.NEXTVAL, p_inspection_date, p_passed, p_collaboration);
        p_return_value := 'OK';
	-- Insert a non-passed inspection
    ELSIF p_passed = 0 THEN
        INSERT INTO inspections(idInspection, inspection_date, passed, collaboration)
        VALUES(seq_id_inspections.NEXTVAL, p_inspection_date, p_passed, p_collaboration);
        p_return_value := 'OK';
	-- Returns an error if the value passed is not 0 or 1
    ELSE
        p_return_value := 'KO: El valor que se ha pasado no es correcto.';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_new_inspection', p_entry_values, p_return_value);  
	RSP := 'insert_new_inspection(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;	
END;
/

-- This procedure is in charge to insert the final report
CREATE OR REPLACE PROCEDURE insert_final_report (
	p_report_date IN DATE,
	p_valoration IN NUMBER,
	p_suitable IN NUMBER,
	p_collaboration IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_collaboration_count NUMBER;
	p_final_report_count NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_report_date || ', ' || p_valoration || ', ' || 
                    p_suitable || ', ' || p_collaboration;
	-- This SELECT COUNT(*) will be used to check if the passed parameters are ok
    SELECT COUNT(*) INTO p_collaboration_count FROM collaborations WHERE idCollaboration = p_collaboration;
	SELECT COUNT(*) INTO p_final_report_count FROM final_reports FR
		INNER JOIN collaborations C ON FR.collaboration = C.idCollaboration
		WHERE C.idCollaboration = p_collaboration;
	-- If the collaboration doesn't exist, return an error
    IF p_collaboration_count = 0 THEN
        p_return_value := 'KO: La colaboración indicada no es correcta.';
	-- If the collaboration has a Final Report, returns an error.
	ELSIF p_final_report_count > 0 THEN
		p_return_value := 'KO: La colaboración indicada ya tiene un informe final.';
	-- Set a suitable final report
    ELSIF p_suitable = 1 THEN
        INSERT INTO final_reports(idReport, report_date, valoration, suitable, collaboration)
        VALUES(seq_id_final_reports.NEXTVAL, p_report_date, p_valoration, p_suitable, p_collaboration);
        p_return_value := 'OK';
	-- Set a non-suitable final report
    ELSIF p_suitable = 0 THEN
        INSERT INTO final_reports(idReport, report_date, valoration, suitable, collaboration)
        VALUES(seq_id_final_reports.NEXTVAL, p_report_date, p_valoration, p_suitable, p_collaboration);
        p_return_value := 'OK';
	-- If the passed value is not 0 or 1, return an error
    ELSE
        p_return_value := 'KO: El valor pasado no es correcto.';
    END IF;
	-- Save the log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_final_report', p_entry_values, p_return_value);
	RSP := 'insert_final_report(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;	
END;
/

-- This stored procedure is in charge to save a new comment to the final report
CREATE OR REPLACE PROCEDURE insert_new_final_report_comment (
    p_final_report_comment IN VARCHAR2,
    p_final_report IN VARCHAR2,
    p_person IN VARCHAR2,
    RSP OUT VARCHAR2
)
AS
    p_final_report_count NUMBER;
    p_person_count NUMBER;
	p_person_count_2 NUMBER;
    p_entry_values VARCHAR(500);
	p_return_value VARCHAR(500);
BEGIN
    p_entry_values := p_final_report_comment || ', ' || p_final_report || ', ' || p_person;
	-- This SELECT COUNT(*) will be used to check if the passed parameters are ok
    SELECT COUNT(*) INTO p_final_report_count FROM final_reports WHERE idReport = p_final_report;
    SELECT COUNT(*) INTO p_person_count FROM persons WHERE dni = p_person;
	SELECT COUNT(*) INTO p_person_count_2 FROM collaborations C
		INNER JOIN final_reports FR ON FR.collaboration = C.idCollaboration
		WHERE (C.student = p_person
			OR C.professor = p_person
			OR C.responsible = p_person)
			AND FR.idReport = p_final_report;
	-- If the final report doesn't exist, return an error.
    IF p_final_report_count = 0 THEN
        p_return_value := 'KO: El informe final indicado no existe.';
	-- If the person doesn't exist, return an error.
    ELSIF p_person_count = 0 THEN
        p_return_value := 'KO: La persona indicada no existe.';
	-- If the person doesn't participate in this collaboration, returns an error.
	ELSIF p_person_count_2 = 0 THEN
		p_return_value := 'KO: La persona indicada no pertenece a este proceso.';
	-- If everything is ok, insert the comment.
    ELSE
        INSERT INTO final_report_comments(idComment, final_report_comment, final_report, person)
        VALUES(seq_id_final_report_comments.NEXTVAL, p_final_report_comment, p_final_report, p_person);
        p_return_value := 'OK';
    END IF;
	-- Save log
    INSERT INTO log_storeds (idLog, log_date, log_process, log_entry, log_return)
    VALUES (seq_id_log_storeds.NEXTVAL, SYSDATE, 'insert_new_final_report_comment', p_entry_values, p_return_value);    
	RSP := 'insert_new_final_report_comment(' || p_entry_values || ') - ' || TO_CHAR(SYSDATE, 'DD-MM-YYYY HH24:MI:SS') || ' - ' || p_return_value;	
END;
/

-- This store procedure receives a DATE and then return the Academic Year (YYYY-YYYY).
CREATE OR REPLACE PROCEDURE check_academic_year (p_date IN DATE, p_academic_year OUT VARCHAR2)
AS
    p_month NUMBER;
    p_year NUMBER;
BEGIN
    SELECT EXTRACT(MONTH FROM p_date) INTO p_month FROM DUAL;
    SELECT EXTRACT(YEAR FROM p_date) INTO p_year FROM DUAL;
	-- Checks if the Date is after or before june to determine the correct academic year
    IF p_month > 6 THEN
        p_academic_year := TO_CHAR(p_year) || '-' || TO_CHAR(p_year + 1);
    ELSE
        p_academic_year := TO_CHAR(p_year - 1) || '-' || TO_CHAR(p_year);
    END IF;
END;
/

-- This stored procedure receives a Date and then insert in diferent tables the new Academic Year.
-- The idea is to execute this procedure only one time every new Academic Year. But can be used if you want to insert data from previous years.
-- If you insert data from previous years without using first this procedure, the Triggers that update the stadistic repository will fail.
CREATE OR REPLACE PROCEDURE insert_new_university_year (p_date IN DATE)
AS
    p_academic_year VARCHAR(20);
BEGIN
	-- Get a new academic_year
    check_academic_year(p_date, p_academic_year); 
    
	-- Insert a new row with the name of the academic_year in three tables.
    INSERT INTO university_year_stats_aux (academic_year)
    VALUES (p_academic_year);
	
    INSERT INTO indicator_type_4 (year)
    VALUES (p_academic_year);
	
	INSERT INTO indicator_type_11 (year)
	VALUES (p_academic_year);
    
	-- Adds a new column with the name of hte year to indicator_11_aux table.
    SELECT CONCAT('year_', REPLACE(p_academic_year, '-', '_')) INTO p_academic_year FROM dual;	
	EXECUTE IMMEDIATE 'ALTER TABLE indicator_11_aux ADD ' || p_academic_year || ' NUMBER DEFAULT 0';   
END;
/

-- General trigger to add companies
CREATE OR REPLACE TRIGGER add_company_trigger
AFTER INSERT ON companies
FOR EACH ROW
BEGIN
	-- Creates a new row in both tables with the cif of the company
    INSERT INTO companies_stats_aux (cif)
    VALUES (:new.cif);
	
	INSERT INTO indicator_11_aux (cif)
	VALUES (:new.cif);
	
	-- Increases the general counter company_counter in one
	UPDATE general_counters
	SET counter_value = counter_value + 1
	WHERE counter_name = 'company_counter';
END;
/

-- General trigger to add universities
CREATE OR REPLACE TRIGGER add_university_trigger
AFTER INSERT ON universities
FOR EACH ROW
BEGIN
	-- Increases the general counter university_counter in one
	UPDATE general_counters
	SET counter_value = counter_value + 1
	WHERE counter_name = 'university_counter';
	
	-- Insert a new row with the cif of the univeristy
	INSERT INTO university_stats_aux (cif)
	VALUES (:new.cif);
END;
/

-- General trigger to add degrees
CREATE OR REPLACE TRIGGER add_degree_trigger
AFTER INSERT ON degrees
FOR EACH ROW
BEGIN
	-- Insert a new row with the code of the degree
	INSERT INTO degree_stats_aux (code)
	VALUES (:new.code);
END;
/

-- General trigger to add professors
CREATE OR REPLACE TRIGGER add_professor_trigger
AFTER INSERT ON professors
FOR EACH ROW
BEGIN
	-- Insert a new row with the dni of the professor
	INSERT INTO professor_stats_aux (dni)
	VALUES (:new.dni);
END;
/

-- This triggers updates the first indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat1_trigger
AFTER INSERT ON final_reports
FOR EACH ROW
DECLARE
	v_university VARCHAR(9);
	v_new_value VARCHAR(9);
	v_old_value VARCHAR(9);
BEGIN
	SELECT P.university INTO v_university FROM collaborations C
    INNER JOIN professors P ON P.dni = C.professor
	WHERE :new.collaboration = C.idCollaboration;
	
	IF :new.suitable = 1 THEN
		UPDATE university_stats_aux
		SET total_collaborations_ended = total_collaborations_ended + 1,
            suitable_collaborations = suitable_collaborations + 1,
			suitable_percentage = ((suitable_collaborations+1)/(total_collaborations_ended+1))*100
        WHERE cif = v_university;
	ELSE
        UPDATE university_stats_aux
        SET total_collaborations_ended = total_collaborations_ended + 1,
            non_suitable_collaborations = non_suitable_collaborations + 1,
			suitable_percentage = (suitable_collaborations/(total_collaborations_ended+1))*100
        WHERE cif = v_university;
    END IF;
	
	SELECT U.type_value INTO v_old_value FROM unique_indicators U
	WHERE U.type_indicator = 1;
	
	SELECT * INTO v_new_value 
	FROM (
		SELECT C.cif FROM university_stats_aux C
		ORDER BY C.suitable_percentage DESC
	) WHERE ROWNUM = 1;
	
	IF (v_new_value != v_old_value) OR (v_old_value IS NULL) THEN	
		UPDATE unique_indicators
		SET type_value = v_new_value
		WHERE type_indicator = 1;	
	END IF;
END;
/

-- This triggers updates the second indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat2_trigger
AFTER INSERT ON offers
FOR EACH ROW
BEGIN
	UPDATE companies_stats_aux
	SET total_offers = total_offers + 1
	WHERE cif = :new.company;
    
    DELETE FROM indicator_type_2;
    INSERT INTO indicator_type_2
    SELECT * FROM (
        SELECT C.cif FROM companies_stats_aux C
        WHERE C.total_offers > 0
        ORDER BY C.total_offers DESC
    )
    WHERE ROWNUM < 6;
END;
/

-- This triggers updates the third and sixtenth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat_3_and_16_trigger
AFTER INSERT ON inspections
FOR EACH ROW
DECLARE 
	v_company VARCHAR(9);
	v_dividend NUMBER;
	v_divisor NUMBER;
	v_percentage NUMBER(5,2);
BEGIN
    SELECT R.company INTO v_company FROM collaborations C
    INNER JOIN responsibles R ON R.dni = C.responsible
    WHERE :new.collaboration = C.idCollaboration;

    UPDATE companies_stats_aux
    SET total_inspections = total_inspections + 1
    WHERE cif = v_company;
	
	IF :new.passed = 1 THEN
		UPDATE companies_stats_aux
		SET passed_inspections = passed_inspections + 1
		WHERE cif = v_company;
	ELSE
		UPDATE companies_stats_aux
		SET failed_inspections = failed_inspections + 1
		WHERE cif = v_company;
	END IF;
	
	SELECT C.failed_inspections INTO v_dividend FROM companies_stats_aux C
	WHERE cif = v_company;
	
	SELECT C.total_inspections INTO v_divisor FROM companies_stats_aux C
	WHERE cif = v_company;
	
	v_percentage := (v_dividend/v_divisor) * 100;
	
	UPDATE companies_stats_aux
	SET failed_inspections_percentage = v_percentage
	WHERE cif = v_company;
    
    SELECT * INTO v_company
    FROM (
        SELECT C.cif FROM companies_stats_aux C
        ORDER BY C.total_inspections DESC
    ) WHERE ROWNUM = 1;
    
    UPDATE unique_indicators
    SET type_value = v_company
    WHERE type_indicator = 3;
	
	DELETE FROM indicator_type_16;
    INSERT INTO indicator_type_16
    SELECT * FROM (
        SELECT C.cif FROM companies_stats_aux C
        WHERE C.failed_inspections_percentage > 10
        ORDER BY C.failed_inspections_percentage DESC
    );
END;
/

-- This triggers updates the fourth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat4_trigger
AFTER INSERT ON final_reports
FOR EACH ROW
DECLARE
    v_academic_year VARCHAR(10);
    v_value NUMBER;
BEGIN
    check_academic_year(:new.report_date , v_academic_year);
    IF :new.suitable = 1 THEN
        UPDATE university_year_stats_aux
        SET total_collaborations_ended = total_collaborations_ended + 1,
		suitable_collaborations = suitable_collaborations + 1,
        non_suitable_percentage = (non_suitable_collaborations/(total_collaborations_ended+1))*100
        WHERE academic_year = v_academic_year;
    ELSE
        UPDATE university_year_stats_aux
        SET total_collaborations_ended = total_collaborations_ended + 1,
            non_suitable_collaborations = non_suitable_collaborations + 1,
            non_suitable_percentage = ((non_suitable_collaborations+1)/(total_collaborations_ended+1))*100
        WHERE academic_year = v_academic_year;
    END IF;
    
    SELECT non_suitable_percentage INTO v_value FROM university_year_stats_aux
    WHERE academic_year = v_academic_year;
    
    UPDATE indicator_type_4
    SET value = v_value
    WHERE year = v_academic_year;
END;
/

-- This triggers updates the fift indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat5_trigger
AFTER INSERT ON collaborations
FOR EACH ROW
DECLARE
	v_academic_year VARCHAR(10);
	v_current_academic_year VARCHAR(10);
    v_value NUMBER(5,2);
BEGIN
	check_academic_year(:new.init_date , v_academic_year);
	check_academic_year(SYSDATE , v_current_academic_year);
	
	UPDATE university_year_stats_aux
	SET total_collaborations = total_collaborations + 1,
        aggregated_income = aggregated_income + :new.remuneration,
        mean_income = ((aggregated_income + :new.remuneration)/(total_collaborations + 1))
	WHERE academic_year = v_academic_year;
    
    IF v_academic_year = v_current_academic_year THEN
        SELECT mean_income INTO v_value FROM university_year_stats_aux
        WHERE academic_year = v_academic_year;
        UPDATE unique_indicators
        SET type_value = TO_CHAR(v_value)
        WHERE type_indicator = 5;
    END IF;
END;
/

-- This triggers updates the sixth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat6_trigger1
AFTER INSERT ON collaborations
FOR EACH ROW
DECLARE
    v_old_value NUMBER;
BEGIN

    SELECT type_value INTO v_old_value FROM unique_indicators
    WHERE type_indicator = 6;
    
    IF v_old_value IS NULL THEN
        UPDATE unique_indicators
        SET type_value = 1
        WHERE type_indicator = 6;
    ELSE
        UPDATE unique_indicators
        SET type_value = type_value + 1
        WHERE type_indicator = 6;
    END IF;
END;
/

-- This triggers updates the sixth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat6_trigger2
AFTER INSERT ON final_reports
FOR EACH ROW
BEGIN    
    UPDATE unique_indicators
    SET type_value = type_value - 1
    WHERE type_indicator = 6;    
END;
/

-- This triggers updates the seventh indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat7_trigger
AFTER INSERT ON collaborations
FOR EACH ROW
DECLARE
    v_university VARCHAR(9);
    v_university_has_collaboration NUMBER(1);
    v_academic_year VARCHAR(10);
    v_current_academic_year VARCHAR(10);
    v_new_value NUMBER(5,2);
    v_dividend NUMBER;
    v_divisor NUMBER;
BEGIN
	check_academic_year(SYSDATE, v_current_academic_year);
	check_academic_year(:new.init_date, v_academic_year);
	IF v_current_academic_year = v_academic_year THEN
		SELECT P.university INTO v_university FROM professors P
		WHERE P.dni = :new.professor;		
		SELECT has_collaboration_this_year INTO v_university_has_collaboration FROM university_stats_aux U
		WHERE U.cif = v_university;
		
		IF v_university_has_collaboration = 0 THEN
			UPDATE university_stats_aux
			SET has_collaboration_this_year = 1
			WHERE cif = v_university;
			
			UPDATE general_counters
			SET counter_value = counter_value + 1
			WHERE counter_name = 'counter_uni_collab_this_year';
			
			SELECT G.counter_value INTO v_dividend FROM general_counters G
            WHERE G.counter_name = 'counter_uni_collab_this_year';
			
			SELECT G.counter_value INTO v_divisor FROM general_counters G
            WHERE G.counter_name = 'university_counter';
			
			v_new_value := ((1 - (v_dividend / v_divisor)) * 100);
            
            UPDATE unique_indicators
            SET type_value = TO_CHAR(v_new_value)
            WHERE type_indicator = 7;
		END IF;
	END IF;  
END;
/

-- This triggers updates the eight indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat8_trigger
AFTER INSERT ON final_reports
FOR EACH ROW
DECLARE
    v_new_code VARCHAR(20);
    v_old_code VARCHAR(20);
    v_old_value NUMBER;
    v_new_value NUMBER;
BEGIN
    IF :new.suitable = 1 THEN
        SELECT S.degree INTO v_new_code FROM Collaborations C
        INNER JOIN students S ON C.student = S.dni
        WHERE C.idCollaboration = :new.collaboration;
        
        UPDATE degree_stats_aux
        SET suitable_collaborations_ended = suitable_collaborations_ended + 1
        WHERE code = v_new_code;
        
        SELECT U.type_value INTO v_old_code FROM unique_indicators U
        WHERE U.type_indicator = 8;
        
        IF v_new_code != v_old_code THEN
            SELECT D.suitable_collaborations_ended INTO v_old_value FROM degree_stats_aux D
            WHERE D.code = v_old_code;
            
            SELECT D.suitable_collaborations_ended INTO v_new_value FROM degree_stats_aux D
            WHERE D.code = v_new_code;         
            
            IF v_new_value > v_old_value THEN
                UPDATE unique_indicators
                SET type_value = v_new_code
                WHERE type_indicator = 8;
            END IF;
		ELSIF v_old_code IS NULL THEN
			SELECT D.suitable_collaborations_ended INTO v_new_value FROM degree_stats_aux D
            WHERE D.code = v_new_code;  
			
			UPDATE unique_indicators
			SET type_value = v_new_code
			WHERE type_indicator = 8;
        END IF;        
    END IF;
END;
/

-- This triggers updates the ninth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat9_trigger1
AFTER INSERT ON offers
FOR EACH ROW
DECLARE
    v_academic_year VARCHAR(10);
    v_current_academic_year VARCHAR(10);
    v_new_value NUMBER(10,2);
    v_dividend NUMBER;
    v_divisor NUMBER;
BEGIN
    check_academic_year(:new.publish_date, v_academic_year);
    
    UPDATE university_year_stats_aux
    SET total_offers = total_offers + 1
    WHERE v_academic_year = academic_year;    
    
    check_academic_year(SYSDATE, v_current_academic_year);
    
    IF v_current_academic_year = v_academic_year THEN
        SELECT U.total_candidacies INTO v_dividend FROM university_year_stats_aux U
        WHERE U.academic_year = v_academic_year;
        
        SELECT U.total_offers INTO v_divisor FROM university_year_stats_aux U
        WHERE U.academic_year = v_academic_year;
        
        v_new_value := v_dividend / v_divisor;
        
        UPDATE unique_indicators
        SET type_value = v_new_value
        WHERE type_indicator = 9;
    END IF;
END;
/

-- This triggers updates the ninth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat9_trigger2
AFTER INSERT ON candidacies
FOR EACH ROW
DECLARE
    v_academic_year VARCHAR(10);
    v_date DATE;
    v_current_academic_year VARCHAR(10);
    v_new_value NUMBER(10,2);
    v_dividend NUMBER;
    v_divisor NUMBER;
BEGIN
    SELECT O.publish_date INTO v_date FROM offers O
    WHERE :new.offer = O.idOffer;
    
    check_academic_year(v_date, v_academic_year);    
    
    UPDATE university_year_stats_aux
    SET total_candidacies = total_candidacies + 1
    WHERE v_academic_year = academic_year;
    
    check_academic_year(SYSDATE, v_current_academic_year);
    
    IF v_current_academic_year = v_academic_year THEN
        SELECT U.total_candidacies INTO v_dividend FROM university_year_stats_aux U
        WHERE U.academic_year = v_academic_year;
        
        SELECT U.total_offers INTO v_divisor FROM university_year_stats_aux U
        WHERE U.academic_year = v_academic_year;
        
        v_new_value := v_dividend / v_divisor;
        
        UPDATE unique_indicators
        SET type_value = v_new_value
        WHERE type_indicator = 9;
    END IF;
END;
/

-- This triggers updates the tenth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat10_trigger
AFTER INSERT ON collaborations
FOR EACH ROW
DECLARE
    v_new_value NUMBER(10,2);
    v_divisor NUMBER;
    v_dividend NUMBER;
BEGIN
	UPDATE general_counters
	SET counter_value = counter_value + 1
	WHERE counter_name = 'collaborations_counter';
	
	UPDATE general_counters
	SET counter_value = counter_value + :new.duration
	WHERE counter_name = 'aggregated_duration';
    
    SELECT G.counter_value INTO v_dividend FROM general_counters G
    WHERE G.counter_name = 'aggregated_duration';
    
    SELECT G.counter_value INTO v_divisor FROM general_counters G
    WHERE G.counter_name = 'collaborations_counter';
    
    v_new_value := v_dividend / v_divisor;
    
    UPDATE unique_indicators
    SET type_value = v_new_value
    WHERE type_indicator = 10;
END;
/

-- This triggers updates the eleventh indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat11_trigger
AFTER INSERT ON interviews
FOR EACH ROW
DECLARE
    v_academic_year VARCHAR(10);
    v_academic_year_column VARCHAR(20);
    v_company VARCHAR(9);
    v_new_value VARCHAR(9);
BEGIN
    SELECT R.company INTO v_company FROM RRHH_contacts R
    WHERE R.dni = :new.rrhh_contact;
    
    check_academic_year(:new.interview_date, v_academic_year);
    SELECT CONCAT('year_', REPLACE(v_academic_year, '-', '_')) INTO v_academic_year_column FROM dual;
    
    EXECUTE IMMEDIATE 'UPDATE indicator_11_aux SET ' || v_academic_year_column || ' = ' || v_academic_year_column || ' + 1 WHERE cif = ''' || v_company || '''';    
    EXECUTE IMMEDIATE 'SELECT * FROM ( SELECT I.cif FROM indicator_11_aux I ORDER BY I.' || v_academic_year_column || ' DESC ) WHERE ROWNUM = 1' INTO v_new_value;
    
    UPDATE indicator_type_11
    SET company = v_new_value
    WHERE year = v_academic_year;
END;
/

-- This triggers updates the twelfth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat12_trigger
AFTER INSERT ON collaborations
FOR EACH ROW
DECLARE
    v_academic_year VARCHAR(10);
    v_current_academic_year VARCHAR(10);
    v_new_value VARCHAR(9);
BEGIN
    check_academic_year(:new.init_date, v_academic_year);
    check_academic_year(SYSDATE, v_current_academic_year);    
    
    IF v_academic_year = v_current_academic_year THEN
		UPDATE professor_stats_aux
		SET current_year_collabs_supervised = current_year_collabs_supervised + 1
		WHERE dni = :new.professor;
		
        SELECT * INTO v_new_value FROM (
            SELECT P.dni FROM professor_stats_aux P
            ORDER BY P.current_year_collabs_supervised DESC
        ) WHERE ROWNUM = 1;
        
        UPDATE unique_indicators
        SET type_value = v_new_value
        WHERE type_indicator = 12;
    END IF;
END;
/

-- This triggers updates the thirteenth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat13_trigger1
AFTER INSERT ON collaborations
FOR EACH ROW
DECLARE
    v_old_value NUMBER;
BEGIN

    SELECT type_value INTO v_old_value FROM unique_indicators
    WHERE type_indicator = 13;
    
    IF v_old_value IS NULL THEN
        UPDATE unique_indicators
        SET type_value = 1
        WHERE type_indicator = 13;
    ELSE
        UPDATE unique_indicators
        SET type_value = type_value + 1
        WHERE type_indicator = 13;
    END IF;
END;
/

-- This triggers updates the thirteenth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat13_trigger2
AFTER INSERT ON final_reports
FOR EACH ROW
BEGIN    
    UPDATE unique_indicators
    SET type_value = type_value - 1
    WHERE type_indicator = 13;    
END;
/

-- This triggers updates the fourteenth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat14_trigger
AFTER INSERT ON offers
FOR EACH ROW
DECLARE
	v_current_academic_year VARCHAR(10);
	v_past_academic_year VARCHAR(10);
	v_past_year DATE;
	v_offer_academic_year VARCHAR(10);
	v_current_check_value NUMBER;
	v_past_check_value NUMBER;
	v_company VARCHAR(9);
	v_dividend NUMBER;
	v_divisor NUMBER;
	v_new_value NUMBER(5,2);
BEGIN
	SELECT ADD_MONTHS(SYSDATE, -12) INTO v_past_year FROM DUAL;
	check_academic_year(SYSDATE, v_current_academic_year); 
	check_academic_year(v_past_year, v_past_academic_year); 
	check_academic_year(:new.publish_date, v_offer_academic_year);
	
	IF v_offer_academic_year = v_past_academic_year THEN
		SELECT C.had_offers_last_year INTO v_past_check_value FROM companies_stats_aux C
		WHERE C.cif = :new.company;
		IF v_past_check_value = 0 THEN
			UPDATE companies_stats_aux
			SET had_offers_last_year = 1
			WHERE cif = :new.company;
			
			UPDATE general_counters
			SET counter_value = counter_value + 1
			WHERE counter_name = 'comp_with_off_last';
			
			SELECT C.has_offers_this_year INTO v_current_check_value FROM companies_stats_aux C
			WHERE C.cif = :new.company;			
			IF v_current_check_value = 0 THEN				
				UPDATE general_counters
				SET counter_value = counter_value + 1
				WHERE counter_name = 'comp_with_off_last_not_current';		
			END IF;
		END IF;		
	ELSIF v_offer_academic_year = v_current_academic_year THEN
		SELECT C.has_offers_this_year INTO v_current_check_value FROM companies_stats_aux C
		WHERE C.cif = :new.company;
		IF v_current_check_value = 0 THEN
			UPDATE companies_stats_aux
			SET has_offers_this_year = 1
			WHERE cif = :new.company;
			
			UPDATE general_counters
			SET counter_value = counter_value + 1
			WHERE counter_name = 'comp_with_off_current';
			
			SELECT C.had_offers_last_year INTO v_past_check_value FROM companies_stats_aux C
			WHERE C.cif = :new.company;
			IF v_past_check_value = 1 THEN			
				UPDATE general_counters
				SET counter_value = counter_value - 1
				WHERE counter_name = 'comp_with_off_last_not_current';			
			END IF;
		END IF;
	END IF;
	
	SELECT G.counter_value INTO v_dividend FROM general_counters G
	WHERE counter_name = 'comp_with_off_last_not_current';
	SELECT G.counter_value INTO v_divisor FROM general_counters G
	WHERE counter_name = 'company_counter';
	v_new_value := (v_dividend / v_divisor) * 100;
	
	UPDATE unique_indicators
	SET type_value = TO_CHAR(v_new_value)
	WHERE type_indicator = 14;	
END;
/

-- This triggers updates the fifteenth indicator and its related counters and pre-calculated variables
CREATE OR REPLACE TRIGGER stat15_trigger
AFTER INSERT ON final_reports
FOR EACH ROW
DECLARE
    v_company VARCHAR(9);
BEGIN
    SELECT R.company INTO v_company FROM collaborations C
    INNER JOIN responsibles R ON R.dni = C.responsible
    WHERE :new.collaboration = C.idCollaboration;
    
    IF :new.suitable = 1 THEN
        UPDATE companies_stats_aux
        SET total_collaborations_ended = total_collaborations_ended + 1,
            suitable_collaborations = suitable_collaborations + 1,
			non_suitable_percentage = (non_suitable_collaborations/(total_collaborations_ended+1))*100
        WHERE cif = v_company;
    ELSE
        UPDATE companies_stats_aux
        SET total_collaborations_ended = total_collaborations_ended + 1,
            non_suitable_collaborations = non_suitable_collaborations + 1,
			non_suitable_percentage = ((non_suitable_collaborations+1)/(total_collaborations_ended+1))*100
        WHERE cif = v_company;
    END IF;
    
    DELETE FROM indicator_type_15;
    INSERT INTO indicator_type_15
    SELECT * FROM (
        SELECT C.cif FROM companies_stats_aux C
        WHERE C.total_collaborations_ended > 0
        ORDER BY C.non_suitable_percentage DESC
    )
    WHERE ROWNUM < 11;
END;
/

CREATE OR REPLACE PROCEDURE new_academic_year_starting
AS
    p_academic_year VARCHAR(20);
    p_value NUMBER;
BEGIN
    -- Get the new academic year
    check_academic_year(SYSDATE, p_academic_year);
    
    -- Reset General Counters
    -- Take the current value of Universities With Offers in Current Year
    SELECT G.counter_value INTO p_value FROM general_counters G
    WHERE counter_name = 'comp_with_off_current';
    -- Put this value in the counter about past year.
    UPDATE general_counters
    SET counter_value = p_value
    WHERE counter_name IN ('comp_with_off_last', 'comp_with_off_last_not_current');
    -- Reset the counter about current year
    UPDATE general_counters
    SET counter_value = 0
    WHERE counter_name = 'comp_with_off_current';
    
    -- Reset Counters About Companies
    UPDATE companies_stats_aux
    SET had_offers_last_year = has_offers_this_year,
        has_offers_this_year = 0;
        
    -- Reset counters About Universities
    UPDATE university_stats_aux
    SET has_collaboration_this_year = 0;
    
    -- Reset counters about professors
    UPDATE professor_stats_aux
    SET current_year_collabs_supervised = 0;
    
    -- Insert new Rows and Columns about the next current year
    insert_new_university_year(SYSDATE);
END;
/

-- This code schedules that every July 1st the counters about current_year 
-- will be reset and new rows and columns about this new year will be added
BEGIN
    DBMS_SCHEDULER.CREATE_JOB (
        job_name => 'new_academic_year_starting_job',
        job_type => 'PLSQL_BLOCK',
        job_action => 'BEGIN new_academic_year_starting; END;',
        start_date => '01-07-2023 00:00:00',
        repeat_interval => 'FREQ=YEARLY;BYMONTH=7;BYMONTHDAY=1',
        enabled => TRUE
    );
END;
/