-- Stat 1
SELECT cif  FROM (    
    SELECT U.cif, U.name, (SUM(F.suitable) / COUNT(F.suitable)) AS percentage FROM final_reports F
    INNER JOIN collaborations C ON C.idCollaboration = F.collaboration
    INNER JOIN students S ON S.dni = C.student
    INNER JOIN universities U ON U.cif = S.university
    GROUP BY U.cif, U.name
    ORDER BY percentage DESC
)
WHERE ROWNUM <= 1;
/

-- Stat 2
SELECT r_cif FROM (
    SELECT C.cif AS r_cif FROM offers O
    INNER JOIN companies C ON C.cif = O.company
    GROUP BY C.name, C.cif
    ORDER BY COUNT(O.idoffer) DESC
)WHERE ROWNUM <= 5;
/

-- Stat 3
SELECT cif FROM (
    SELECT Co.cif, Co.name, count(I.idInspection) FROM inspections I
    INNER JOIN collaborations C ON C.idCollaboration = I.collaboration
    INNER JOIN responsibles R ON C.responsible = R.dni
    INNER JOIN companies Co ON Co.cif = R.company
    GROUP BY Co.cif, Co.name
    ORDER BY count(I.idInspection) DESC
)
WHERE ROWNUM <= 1;
/

-- Stat 4
SELECT TO_CHAR(r_year || '-' || (r_year + 1)) as r_academic_year, (CASE WHEN r_value IS NULL THEN 0 ELSE r_value END) AS r_result FROM (
    SELECT A.r_year, ((B.r_total / A.r_total) * 100) AS r_value FROM(
        SELECT EXTRACT(YEAR FROM C.init_date) - (CASE WHEN EXTRACT(MONTH FROM C.init_date) < 7 THEN 1 ELSE 0 END) AS r_year, COUNT(FR.idReport) AS r_total FROM collaborations C
        INNER JOIN final_reports FR ON C.idCollaboration = FR.collaboration
        GROUP BY EXTRACT(YEAR FROM C.init_date) - (CASE WHEN EXTRACT(MONTH FROM C.init_date) < 7 THEN 1 ELSE 0 END)
    ) A LEFT JOIN (
        SELECT EXTRACT(YEAR FROM C.init_date) - (CASE WHEN EXTRACT(MONTH FROM C.init_date) < 7 THEN 1 ELSE 0 END) AS r_year, COUNT(FR.idReport) AS r_total FROM collaborations C
        INNER JOIN final_reports FR ON C.idCollaboration = FR.collaboration
        WHERE FR.suitable = 0
        GROUP BY EXTRACT(YEAR FROM C.init_date) - (CASE WHEN EXTRACT(MONTH FROM c.init_date) < 7 THEN 1 ELSE 0 END)
    ) B ON A.r_year = B.r_year
);
/

-- Stat 5
SELECT AVG(remuneration) FROM collaborations
WHERE collaborations.init_date > '01-07-2022';
/

-- Stat 6
SELECT (num_c.count - num_fr.count) AS result FROM (
    SELECT COUNT(*) AS count FROM collaborations C
)num_c, (
    SELECT COUNT(*) AS count  FROM final_reports FR
)num_fr;
/

-- Stat 7
SELECT (((B.total - A.total) / B.total)*100) AS result FROM (
    SELECT COUNT(*) as total FROM (
        SELECT U.cif AS value FROM collaborations C
        INNER JOIN students S ON S.dni = C.student
        INNER JOIN universities U ON U.cif = S.university
        WHERE C.init_date > '01-01-2022'
        GROUP BY U.cif
    )
) A, (
    SELECT COUNT(*) AS total FROM universities
) B;
/

-- Stat 8
SELECT Code AS result FROM (
    SELECT D.Code, D.name AS name, COUNT(FR.idReport) AS aproveds FROM final_reports FR
    INNER JOIN collaborations C ON C.idCollaboration = FR.collaboration
    INNER JOIN students S ON S.dni = C.student
    INNER JOIN degrees D ON D.code = S.degree
    WHERE FR.suitable = 1
    GROUP BY D.code, D.name
    ORDER BY aproveds DESC
)
WHERE ROWNUM <= 1;
/

-- Stat 9
SELECT AVG(value) AS result FROM (
    SELECT O.idOffer, COUNT(C.idCandidacy) AS value FROM candidacies C
    RIGHT JOIN offers O ON O.idOffer = C.offer
    WHERE O.publish_date > '01-01-2022'
    GROUP BY O.idOffer
);
/

-- Stat 10
SELECT AVG(duration) AS result FROM collaborations;
/

-- Stat 11 (Esta consulta, a diferencia del disparador oficial, muestra si hay empates en la empresa que más entrevistas ha hecho en un año)
SELECT TO_CHAR(A.r_year || '-' || (A.r_year + 1)) as r_academic_year, A.r_cif FROM (
    SELECT EXTRACT(YEAR FROM I.interview_date) AS r_year, COUNT(I.idInterview) AS r_value, C.cif AS r_cif
    FROM interviews I
    INNER JOIN RRHH_contacts RH ON RH.dni = I.rrhh_contact
    INNER JOIN companies C ON RH.company = C.cif
    GROUP BY EXTRACT(YEAR FROM I.interview_date), C.cif
) A INNER JOIN (
    SELECT r_year, MAX(r_value) AS max_value FROM (
        SELECT EXTRACT(YEAR FROM I.interview_date) AS r_year, COUNT(I.idInterview) AS r_value
        FROM interviews I
        INNER JOIN RRHH_contacts RH ON RH.dni = I.rrhh_contact
        INNER JOIN companies C ON RH.company = C.cif
        GROUP BY EXTRACT(YEAR FROM I.interview_date), C.cif
    ) GROUP BY r_year
) B ON A.r_year = B.r_year AND A.r_value = B.max_value
ORDER BY A.r_year;
/

-- Stat 12
SELECT dni FROM ( 
    SELECT PR.dni, P.name as name, COUNT(C.idCollaboration) AS value FROM collaborations C
    RIGHT JOIN professors PR ON PR.dni = C.professor
    INNER JOIN persons P ON P.dni = PR.dni
    WHERE C.init_date > '01-01-2022'
    GROUP BY PR.dni, P.name
    ORDER BY value DESC
)
WHERE ROWNUM <= 1;
/

-- Stat 13
SELECT (num_c.count - num_fr.count) AS result FROM (
    SELECT COUNT(*) AS count FROM collaborations C
)num_c, (
    SELECT COUNT(*) AS count  FROM final_reports FR
)num_fr;
/

-- Stat 14
SELECT ((A.dividend / B.divisor)*100) AS result FROM ( 
    SELECT COUNT(company) AS dividend FROM (
        SELECT company FROM (
            SELECT company FROM offers
            WHERE offers.publish_date < '30-06-2022'
                AND offers.publish_date > '01-01-2021'
            GROUP BY company
        ) MINUS (
            SELECT company FROM offers
            WHERE offers.publish_date > '01-01-2022'
            GROUP BY company
        )
    )
) A, (
    SELECT COUNT(*) AS divisor FROM companies
) B;
/

-- Stat 15
SELECT r_cif FROM (
    SELECT r_cif, (CASE WHEN r_value IS NULL THEN 0 ELSE r_value END) AS r_percentage FROM (
        SELECT B.r_cif, ((A.r_value / B.r_value) * 100) AS r_value FROM (
            SELECT CO.cif AS r_cif, COUNT(FR.idReport) AS r_value FROM final_reports FR
            INNER JOIN collaborations C ON FR.collaboration = C.idCollaboration
            INNER JOIN responsibles R ON C.responsible = R.dni
            INNER JOIN companies CO ON R.company = CO.cif
            WHERE FR.suitable = 0
            GROUP BY CO.cif
        ) A RIGHT JOIN (
            SELECT CO.cif AS r_cif, COUNT(FR.idReport) AS r_value FROM final_reports FR
            INNER JOIN collaborations C ON FR.collaboration = C.idCollaboration
            INNER JOIN responsibles R ON C.responsible = R.dni
            INNER JOIN companies CO ON R.company = CO.cif
            GROUP BY CO.cif
        )B ON A.r_cif = B.r_cif
    ) ORDER BY r_percentage DESC
)WHERE ROWNUM <= 10;
/

-- Stat 16
SELECT p_cif FROM (
    SELECT p_cif, (CASE WHEN p_value IS NULL THEN 0 ELSE p_value END) AS p_percentage FROM (
        SELECT B.p_cif, ((A.p_value / B.p_value) * 100) AS p_value FROM (
            SELECT CO.cif AS p_cif, COUNT(I.idInspection) AS p_value FROM inspections I
            INNER JOIN collaborations C ON I.collaboration = C.idCollaboration
            INNER JOIN responsibles R ON C.responsible = R.dni
            INNER JOIN companies CO ON R.company = CO.cif
            WHERE I.passed = 0
            GROUP BY CO.cif
        ) A RIGHT JOIN (
            SELECT CO.cif AS p_cif, COUNT(I.idInspection) AS p_value FROM inspections I
            INNER JOIN collaborations C ON I.collaboration = C.idCollaboration
            INNER JOIN responsibles R ON C.responsible = R.dni
            INNER JOIN companies CO ON R.company = CO.cif
            GROUP BY CO.cif
        )B ON A.p_cif = B.p_cif
    ) ORDER BY p_percentage DESC
) WHERE p_percentage > 10;
/